/*
 * Copyright (C) 2011 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Alex Launi <alex.launi@canonical.com>
 *
 */

using GLib;

namespace Unity.MusicLens {

  public class MusicStoreCollection : Object
  {

    private const string MUSICSTORE_BASE_URI = "http://musicsearch.ubuntu.com/v1/";
    private const uint PURCHASE_CATEGORY = 2;

    public MusicStoreCollection ()
    {
    }

    public async void search (LensSearch search, SearchType search_type,
			owned List<FilterParser> filters, int max_results = -1, Cancellable cancellable) throws IOError
    {
      string? uri = build_search_uri (search.search_string, filters);

      if (uri == null)
        return;	  

      var results_model = search.results_model;
      File file = File.new_for_uri (uri);

      yield read_musicstore_search_result_async (file, results_model, cancellable);
    }

    private async void read_musicstore_search_result_async (File file, Dee.Model model, Cancellable cancellable)
    {
      var timer = new Timer ();
      debug ("Searching %s", file.get_uri ());

      try {
        var stream = yield file.read_async (Priority.DEFAULT, cancellable);
	var dis = new DataInputStream (stream);
	var parser = new Json.Parser ();
	yield parser.load_from_stream_async (dis, cancellable);
	var root_object = parser.get_root ().get_object ();
	
	Json.Object? results = root_object.get_object_member ("results");
	if (results == null) {
	  warning ("Invalid response from server. No 'results' member.");
	  return;
	}
	
	var albums = results.get_array_member ("album").get_elements ();
	var tracks = results.get_array_member ("track").get_elements ();
	
	debug ("Got %u albums and %u tracks", albums.length (), tracks.length ());
	
	unowned string? uri, artwork_path, mimetype, title, artist, dnd_uri;
	
	foreach (var album_node in albums) {
	  var album_obj = album_node.get_object ();
	  
	  uri = album_obj.get_string_member ("purchase_url");
	  artwork_path = album_obj.get_string_member ("image");
	  mimetype = "audio-x-generic";
	  title = album_obj.get_string_member ("title");
	  artist = album_obj.get_string_member ("artist");
	  dnd_uri = uri;
	  
	  model.append (uri, artwork_path, PURCHASE_CATEGORY,
	                mimetype, title, artist, dnd_uri);
        }
        
	foreach (var track_node in tracks) {
	  var track_obj = track_node.get_object ();
	  
	  uri = track_obj.get_string_member ("purchase_url");
	  artwork_path = track_obj.get_string_member ("image");
	  mimetype = "audio-x-generic";
	  title = track_obj.get_string_member ("title");
	  artist = track_obj.get_string_member ("artist");
	  dnd_uri = uri;
          
	  // FIXME drag n drop uri needs to be the u1ms:// link
	  
	  model.append (uri, artwork_path, PURCHASE_CATEGORY,
	                mimetype, title, artist, dnd_uri);
        }        

	debug ("Retrieved '%s' in %fms", file.get_uri (), timer.elapsed()*1000);
	debug ("Model has %u rows after search", model.get_n_rows ());

      } catch (Error e) {
	warning ("Error reading URL '%s': %s", file.get_uri (), e.message);
      }
    }

    private string? build_search_uri (string query, List<FilterParser> filters)
    {
      if (query.strip() == "")
        return null;
    
      MusicStoreFilterParser store_parser;
      string musicstore_base_uri = MUSICSTORE_BASE_URI;
      if (GLib.Environment.get_variable("MUSICSTORE_URI") != null)
        musicstore_base_uri = GLib.Environment.get_variable("MUSICSTORE_URI");
      debug ("Using base URI of '%s'", musicstore_base_uri);
      StringBuilder uri = new StringBuilder (musicstore_base_uri);
      uri.append ("search?q=");
      uri.append (Uri.escape_string (query, "", false));

      foreach (FilterParser parser in filters) {
	  if (parser is GenreFilterParser)
	    store_parser = new MusicStoreGenreFilterParser (parser as GenreFilterParser);
	  else if (parser is DecadeFilterParser)
	    store_parser = new MusicStoreDecadeFilterParser (parser as DecadeFilterParser);
	  else
	    continue;

	  uri.append (store_parser.parse ());
      }
      
      uri.append ("&pagesize=10");
      
      // This makes the service return $pagesize results *per content type*.
      // Which we need - as it could otherwise return $pagesize results mixed
      // or artist,album, and track. Since we can't display artists, this can
      // lead to an empty result set in the dash if there is only artists in
      // the response from the webservice
      uri.append ("&grouping=1");

      return uri.str;
    }
  }
}
